/*---------------------------------------------------------------------------*/
//       Author : hiyohiyo
//         Mail : hiyohiyo@crystalmark.info
//          Web : http://crystalmark.info/
//      License : The modified BSD license
//
//                                Copyright 2002- hiyohiyo All rights reserved.
/*---------------------------------------------------------------------------*/

#include "stdafx.h"
#include "crystalcpuid.h"
#include "CrystalCPUIDDlg.h"
#include "RealTimeClockDlg.h"
#include "LcdSetting.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static CCrystalCPUIDDlg* P;

CRealTimeClockDlg::CRealTimeClockDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRealTimeClockDlg::IDD, pParent)
{
	P = (CCrystalCPUIDDlg*)pParent;
	//{{AFX_DATA_INIT(CRealTimeClockDlg)
	//}}AFX_DATA_INIT
}


void CRealTimeClockDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRealTimeClockDlg)
	DDX_Control(pDX, IDC_RTC_CLOCK, m_Clock);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRealTimeClockDlg, CDialog)
	//{{AFX_MSG_MAP(CRealTimeClockDlg)
	ON_WM_TIMER()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_RTC_CLOCK, OnRtcClock)
	ON_WM_SHOWWINDOW()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRealTimeClockDlg 

void CRealTimeClockDlg::OnOK() 
{
	CDialog::OnOK();
}

void CRealTimeClockDlg::OnCancel() 
{
	CDialog::OnCancel();
}

BOOL CRealTimeClockDlg::OnInitDialog() 
{
	CString cstr;
	char str[256];

	CDialog::OnInitDialog();

	GetPrivateProfileString("Setting","IntervalTime","1000",str,256,P->m_ini);
	if( atoi(str) <= 0 ){
		m_IntervalTime = 1000;
	}else{
		m_IntervalTime = atoi(str);
	}
	GetPrivateProfileString("Setting","MethodType","0",str,256,P->m_ini);
	if( atoi(str) <= 0 ){
		m_MethodType = 0;
	}else{
		m_MethodType = atoi(str);
	}

	cstr.Format("%s [%s%s%s]",
		P->m_CPUName,
		P->m_Family,
		P->m_Model,
		P->m_Stepping);
	SetWindowText(cstr);

	double clock = atof(P->m_Clock);
	P->SysInfo->GetString(CPU_MULTIPLIER_UPDATE,str);
	double multiplier = atof(str);
	double system = clock / multiplier;

	if( multiplier > 0 ){
		cstr.Format("%9.2fMHz%6.2fx%5.2f",
			clock,
			system,
			multiplier);
	}else{
		cstr.Format("%9.2fMHz%9.2fGHz",
			clock,
			clock / 1000.0);
	}
	m_Clock.SetText(cstr);

	m_Clock.SetNumberOfLines(2);
	m_Clock.SetXCharsPerLine(12);
	m_Clock.SetSize(CMatrixStatic::LARGE);

	DWORD LcdGridRgb,  LcdOnRgb, LcdOffRgb;
	GetPrivateProfileString("Setting","LcdGrid",DEFAULT_LCD_GRID,str,256,P->m_ini);
	if( strcmp(str,"") != 0 ){
		LcdGridRgb = atoi(str);
	}else{
		LcdGridRgb = atoi(DEFAULT_LCD_GRID);
	}

	GetPrivateProfileString("Setting","LcdOn",DEFAULT_LCD_ON,str,256,P->m_ini);
	if( strcmp(str,"") != 0 ){
		LcdOnRgb = atoi(str);
	}else{
		LcdOnRgb = atoi(DEFAULT_LCD_ON);
	}

	GetPrivateProfileString("Setting","LcdOff",DEFAULT_LCD_OFF,str,256,P->m_ini);
	if( strcmp(str,"") != 0 ){
		LcdOffRgb = atoi(str);
	}else{
		LcdOffRgb = atoi(DEFAULT_LCD_OFF);
	}
	m_Clock.SetDisplayColors( LcdGridRgb, LcdOnRgb, LcdOffRgb );

	m_Clock.AdjustClientXToSize(12);
	m_Clock.AdjustClientYToSize(2);
	m_Clock.SetAutoPadding(true);
//	m_Clock.DoScroll(500, CMatrixStatic::RIGHT);

	// Adjust Window Size
	RECT rect,rect1;
	m_Clock.GetWindowRect(&rect);
	GetWindowRect(&rect1);
	MoveWindow(rect1.left,rect1.top,270,rect.bottom - rect1.top - 1);
	CenterWindow();

	GetPrivateProfileString("Setting","TopMost","1",str,256,P->m_ini);
	if( atoi(str) >= 1 ){
		::SetWindowPos(m_hWnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE | SWP_NOSIZE); 
	}

	SetTimer(1, m_IntervalTime, NULL);
	return TRUE;
}

void CRealTimeClockDlg::OnTimer(UINT nIDEvent) 
{
	CString cstr;
	static DWORD dwStart,dwEnd, TickCount;			// GetTickCount()
	static DWORD dwStartH,dwStartL,dwEndH,dwEndL;	// QPC
	static ULARGE_INTEGER lRdtscS,lRdtscE;
	static LARGE_INTEGER lStart,lEnd,lFrequency;
	static BOOL Flag = TRUE;

	if( m_MethodType == 0 ){ // No Load
		if( Flag ){
			QueryPerformanceCounter(&lStart);
			_asm{
				rdtsc
				mov dwStartH , edx
				mov dwStartL , eax
			}
			dwStart = GetTickCount();
			lRdtscS.HighPart = dwStartH;
			lRdtscS.LowPart  = dwStartL;
			Flag = FALSE;
		}else{
			dwEnd = GetTickCount();
			_asm{
				rdtsc
				mov dwEndH , edx
				mov dwEndL , eax
			}
			QueryPerformanceCounter(&lEnd);
			QueryPerformanceFrequency(&lFrequency);

			lRdtscE.HighPart = dwEndH;
			lRdtscE.LowPart  = dwEndL;

			double totalTime;
			double totalTimeQPC = (double)( (lEnd.QuadPart - lStart.QuadPart)) / (double)(lFrequency.QuadPart);
			double totalTimeWT  = (double)(dwEnd - dwStart) / 1000.0;

			const double ERROR_RATE = (double)m_IntervalTime/1000.0/10;
			if(totalTimeWT - ERROR_RATE < totalTimeQPC && totalTimeQPC < totalTimeWT + ERROR_RATE){
				totalTime = totalTimeQPC;
			}else{
				totalTime = totalTimeWT;
			}

			double clock = (double)( (__int64)(lRdtscE.QuadPart - lRdtscS.QuadPart) / totalTime ) / 1000.0 / 1000.0;
			char str[256];
			P->SysInfo->GetString(CPU_MULTIPLIER_UPDATE,str);
			double multiplier = atof(str);
			double system = clock / multiplier;

			if( multiplier > 0 ){
				cstr.Format("%9.2fMHz%6.2fx%5.2f",
					clock,
					system,
					multiplier);
			}else{
				cstr.Format("%9.2fMHz%9.2fGHz",
					clock,
					clock / 1000.0);
			}
			m_Clock.SetText(cstr);

			dwStart = dwEnd;
			lStart = lEnd;
			lRdtscS.HighPart = lRdtscE.HighPart;
			lRdtscS.LowPart = lRdtscE.LowPart;
		}
	}else{ // With Load
		QueryPerformanceCounter(&lStart);
		_asm{
			rdtsc
			mov dwStartH , edx
			mov dwStartL , eax
		}
		dwStart = GetTickCount();
		lRdtscS.HighPart = dwStartH;
		lRdtscS.LowPart  = dwStartL;
		
		volatile int i=0;
		TickCount = GetTickCount();
		while(GetTickCount() - TickCount < 100){
			i++;
			Sleep(0);
		}

		dwEnd = GetTickCount();
		_asm{
			rdtsc
			mov dwEndH , edx
			mov dwEndL , eax
		}
		QueryPerformanceCounter(&lEnd);
		QueryPerformanceFrequency(&lFrequency);

		lRdtscE.HighPart = dwEndH;
		lRdtscE.LowPart  = dwEndL;

		double totalTime;
		double totalTimeQPC = (double)( (lEnd.QuadPart - lStart.QuadPart)) / (double)(lFrequency.QuadPart);
		double totalTimeWT  = (double)(dwEnd - dwStart) / 1000.0;

		const double ERROR_RATE = (double)m_IntervalTime/1000.0/10;
		if(totalTimeWT - ERROR_RATE < totalTimeQPC && totalTimeQPC < totalTimeWT + ERROR_RATE){
			totalTime = totalTimeQPC;
		}else{
			totalTime = totalTimeWT;
		}

		double clock = (double)( (__int64)(lRdtscE.QuadPart - lRdtscS.QuadPart) / totalTime ) / 1000.0 / 1000.0;
		char str[256];
		P->SysInfo->GetString(CPU_MULTIPLIER_UPDATE,str);
		double multiplier = atof(str);
		double system = clock / multiplier;

		if( multiplier > 0 ){
			cstr.Format("%9.2fMHz%6.2fx%5.2f",
				clock,
				system,
				multiplier);
		}else{
			cstr.Format("%9.2fMHz%9.2fGHz",
				clock,
				clock / 1000.0);
		}
		m_Clock.SetText(cstr);

		dwStart = dwEnd;
		lStart = lEnd;
		lRdtscS.HighPart = lRdtscE.HighPart;
		lRdtscS.LowPart = lRdtscE.LowPart;
	}

	CDialog::OnTimer(nIDEvent);
}

void CRealTimeClockDlg::PostNcDestroy() 
{
	CDialog::PostNcDestroy();
}

void CRealTimeClockDlg::OnClose() 
{
	CMenu *menu = P->GetMenu();
	menu->EnableMenuItem(IDM_REAL_TIME_CLOCK,MF_ENABLED);
	P->SetMenu(menu);
	P->ShowWindow(SW_SHOW);

	CDialog::OnClose();
	delete this;
	P->m_RealTimeClockDlg = NULL;
}

BOOL CRealTimeClockDlg::Create()
{
	BOOL bret = CDialog::Create( IDD, P );
	if( bret == TRUE){
		this->ShowWindow(SW_SHOW);
	}
	return bret;
}

void CRealTimeClockDlg::OnRtcClock() 
{
	static int Flag = SW_SHOW;
	if(Flag == SW_SHOW){
		Flag = SW_HIDE;
	}else{
		Flag = SW_SHOW;
	}
	P->ShowWindow(Flag);
}

void CRealTimeClockDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);
	
	P->ShowWindow(SW_SHOW);
}

void CRealTimeClockDlg::ChangeIntervalTime(int time)
{
	m_IntervalTime = time;
	KillTimer(1);
	SetTimer(1, time, NULL);
}

void CRealTimeClockDlg::ChangeMethodType(int method)
{
	m_MethodType = method;
}
